/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// StarRecords implementation
//
// Loads vallongtype.txt into memory allowing searching by multiple
// criteria. This file is a public domain file downloaded from
// AAVSO at http://www.aavso.org

#include <iostream>
#include <cstdlib>
#include <cstring>
#include "CStarRecords.h"
using namespace std;

//--== Default Constructor ==--------------------------------------------------
//
StarRecords::StarRecords()
{
	filename[0] = '\0';
	rptname[0]  = '\0';
	//starFile = NULL;
	//starOut  = NULL;

	topPtr     = NULL;
	lastPtr    = NULL;
	topResPtr  = NULL;
	resPtr     = NULL;
	lastResPtr = NULL;
	is_loaded   = false;
	is_searched = false;
	is_specific = false;
	begin	= false;
	end	= false;

	minHr   = 0;
	maxHr   = 23;
	minDeg  = -90;
	maxDeg  = 90;
	minDays = 0;
	maxDays = 10000;
	minMag  = 6;
	maxMag  = 7;
}

//--== Mutator Functions ==----------------------------------------------------
//
// If values are in acceptable range, change. Otherwise, the constructor
// or a previous set operation has already determined the range.
//
void StarRecords::setSchMinHr  (short int min_ra_hrs)
{
	if(min_ra_hrs >= 0 && min_ra_hrs <= 23)
		minHr = min_ra_hrs;
}
void StarRecords::setSchMaxHr  (short int max_ra_hrs)
{
	if(max_ra_hrs >= 0 && max_ra_hrs <= 23)
		maxHr = max_ra_hrs;
}
void StarRecords::setSchMinDeg (short int min_dec_deg)
{
	if(min_dec_deg >= -90 && min_dec_deg <= 90
		&& min_dec_deg <= maxDeg)
		minDeg = min_dec_deg;
}
void StarRecords::setSchMaxDeg (short int max_dec_deg)
{
	if(max_dec_deg >= -90 && max_dec_deg <= 90
		&& max_dec_deg >= minDeg)
		maxDeg = max_dec_deg;
}
void StarRecords::setSchMinDays(double min_days)
{
	if(min_days >= 0 && min_days <= maxDays)
		minDays = min_days;
}
void StarRecords::setSchMaxDays(double max_days)
{
	if(max_days > 0 && max_days >= minDays)
		maxDays = max_days;
}
void StarRecords::setSchMinMag (double min_mag)
{
	if(min_mag >= 0)
		minMag  = static_cast<float>(min_mag);
}
void StarRecords::setSchMaxMag (double max_mag)
{
	if(max_mag >= 0)
		maxMag  = static_cast<float>(max_mag);
}

//--== Accessor Functions ==---------------------------------------------------
//
// Return values protected by const
//
short int StarRecords::getSchMinHr  () const {return minHr;}
short int StarRecords::getSchMaxHr  () const {return maxHr;}
short int StarRecords::getSchMinDeg () const {return minDeg;}
short int StarRecords::getSchMaxDeg () const {return maxDeg;}
double    StarRecords::getSchMinDays() const {return minDays;}
double    StarRecords::getSchMaxDays() const {return maxDays;}
float     StarRecords::getSchMinMag () const {return minMag;}
float     StarRecords::getSchMaxMag () const {return maxMag;}

//--== loadRecords() ==--------------------------------------------------------
//
// Create master linked list data nodes from passed file name
//
int StarRecords::loadRecords(const char* fname)
{
	// Destroy master linked list if there is one
	if(is_loaded == true)
	{
		delRecords();
		is_loaded = false;
	}

	int i = 0, j, k;

	// Remember the file name
	strcpy(filename, fname);
	starFile.open(filename);

	// Error trap for failed file open
	if(!starFile.is_open())
	{
		starFile.clear();
		starFile.close();
		return -1;	// FAILURE VALUE returns here
	}

	VarStarRecord* recPtr;

	// How long should buffer be?
	char linebuf[256];

	// Executable hangs the load so need addtl test
	while (!starFile.eof())
	{
		starFile.getline(linebuf, 256);

		// If buffer contains a valid record,
		// add a node to the master linked list.
		if(isLineValidRecord(linebuf))
		{
			if(i == 0)
			{
				topPtr = new VarStarRecord;
				recPtr = topPtr;
			}
			else
			{
				recPtr->pNext = new VarStarRecord;
				recPtr = recPtr->pNext;
			}

			// This prevents the CR char in DOS from
			// causing problems with the space padder
			k = static_cast<int>(strlen(linebuf));
			if(linebuf[k-1] == 13)
				k--;

			// Pad short records with spaces, 
			// crop long records to length
			if(k != 75)
			{
				if(k < 75)
					for(j = k; j < 75; j++)
						linebuf[j] = ' ';

				linebuf[75] = '\0';
			}

			// Raw record
			strcpy(recPtr->record, linebuf);
			// Record number
			recPtr->recnum = ++i;
			// R.A. hours
			setHours(recPtr);
			// R.A. minutes
			setMinutes(recPtr);
			// Declination degrees
			setDegrees(recPtr);
			// Name of star
			setName(recPtr);
			// Type of star
			setType(recPtr);
			// Range string of star
			setRange(recPtr);
			// Maximum magnitude
			setMaxMag(recPtr);
			// Minumum magnitude
			setMinMag(recPtr);
			// Period in days
			setDays(recPtr);
			// Desc of star
			setDesc(recPtr);
		}
	}

	// If records were loaded, set vars
	if(i > 0)
	{
		is_loaded = true;
		recPtr->pNext = NULL;
		clearSearch();
	}
	else
		is_loaded = false;

	starFile.clear();
	starFile.close();

	return i;
}

//--== saveReport() ==---------------------------------------------------------
//
// Save current search results as a comma separated file with column headers
//
int StarRecords::saveReport (const char* fname)
{
	if(!is_loaded)
		return 0;

	// ".csv" appended to passed file name
	strcpy(rptname, fname);
	//strcat(rptname, ".csv");
	starOut.open(rptname);

	if(!starOut.is_open())
	{
		starOut.clear();
		starOut.close();
		return -1;	// FAILURE VALUE returns here
	}

	VarStarResult* rPtr = topResPtr;
	int i = 0;

	// This writes out the comma-separated file that is
	// loadable in pretty much any spreadsheet
	starOut << "Hr,Min,Deg,Name,Type,Range,Desc,Max,Min,Period" << endl;
	while(rPtr != NULL)
	{
		starOut
			<< rPtr->pVSR->ra_hours << ','
			<< rPtr->pVSR->ra_minutes << ','
			<< rPtr->pVSR->dec_degrees << ",\""
			<< rPtr->pVSR->name << "\",\""
			<< rPtr->pVSR->type << "\",\""
			<< rPtr->pVSR->range << "\",\""
			<< rPtr->pVSR->desc << "\","
			<< rPtr->pVSR->max_mag << ','
			<< rPtr->pVSR->min_mag << ','
			<< rPtr->pVSR->period_days
			<< endl;

		i++;
		rPtr = rPtr->pNext;
	}

	starOut.clear();
	starOut.close();

	return i;
}

//--== saveFile() ==-----------------------------------------------------------
//
// Save current search results as a text file with records in original format
//
int StarRecords::saveFile(const char* fname)
{
	if(is_loaded == false)
		return 0;

	// ".txt" appended to passed file name
	strcpy(rptname, fname);
	//strcat(rptname, ".txt");
	starOut.open(rptname);

	if(!starOut.is_open())
	{
		starOut.clear();
		starOut.close();
		return -1;	// FAILURE VALUE returns here
	}

	VarStarResult* rPtr = topResPtr;
	int i = 0;

	// Write out raw records to output file
	while(rPtr != NULL)
	{
		starOut << rPtr->pVSR->record << endl;
		i++;
		rPtr = rPtr->pNext;
	}

	starOut.clear();
	starOut.close();

	return i;
}

//--== isLineValidRecord() ==--------------------------------------------------
//
// Test buffer for valid record format
//
bool StarRecords::isLineValidRecord(const char* testStr)
{
	// 4 bytes: R.A. as hhmm
	// 1 byte:  north or south as + or -
	// 2 bytes: dec. in whole degrees
	// 1 byte:  space
	// 1 byte:  alpha character
	// 2 bytes: spaces
	// 2 bytes: printable non-space
	if(!(
		isdigit(testStr[0]) && isdigit(testStr[1]) &&
		isdigit(testStr[2]) && isdigit(testStr[3]) &&
		(
			testStr[4] == '+'   ||
			testStr[4] == '-'
		) &&
		isdigit(testStr[5]) && isdigit(testStr[6]) &&
		(
			isspace(testStr[7]) ||
			isalpha(testStr[7])
		) &&
		isspace(testStr[8])  && isspace(testStr[9]) &&
		isgraph(testStr[10]) && isgraph(testStr[22])
	))
		return false;

	return true;
}

//--== setHours() ==-----------------------------------------------------------
// Parse chars into integer list node member
void StarRecords::setHours(VarStarRecord* vPtr)
{
	vPtr->ra_hours = 0;

	if(isdigit(vPtr->record[0]))
		vPtr->ra_hours +=
			(static_cast<short int>(vPtr->record[0]) - 48) * 10;
	if(isdigit(vPtr->record[1]))
		vPtr->ra_hours +=
			(static_cast<short int>(vPtr->record[1]) - 48);
}

//--== setMinutes() ==---------------------------------------------------------
// Parse chars into integer list node memeber
void StarRecords::setMinutes(VarStarRecord* vPtr)
{
	vPtr->ra_minutes = 0;

	if(isdigit(vPtr->record[2]))
		vPtr->ra_minutes +=
			(static_cast<short int>(vPtr->record[2]) - 48) * 10;
	if(isdigit(vPtr->record[3]))
		vPtr->ra_minutes +=
			(static_cast<short int>(vPtr->record[3]) - 48);
}

//--== setDegrees() ==---------------------------------------------------------
// Parse chars into integer list node member
void StarRecords::setDegrees(VarStarRecord* vPtr)
{
	vPtr->dec_degrees  = 0;

	if(isdigit(vPtr->record[5]))
		vPtr->dec_degrees +=
			(static_cast<short int>(vPtr->record[5]) - 48) * 10;
	if(isdigit(vPtr->record[6]))
		vPtr->dec_degrees +=
			(static_cast<short int>(vPtr->record[6]) - 48);
	if(vPtr->record[4] == '-')
		vPtr->dec_degrees = -(vPtr->dec_degrees);
}

//--== setName() ==------------------------------------------------------------
// Copy name into char* list node member - right aligned & space padded
void StarRecords::setName(VarStarRecord* vPtr)
{
	int i, j = 0;

	// determines number of spaces to pad from the left
	for(i = 20; i >= 10; i--)
	{
		if(vPtr->record[i] == ' ')
			j++;
		else
			break;
	}

	vPtr->name[11] = '\0';

	// fill in the name from the right
	for(i = 10 - j; i >= 0; i--)
		vPtr->name[i + j] = vPtr->record[i + 10];

	// fill in the spaces from the right
	for(i = 0; i < j; i++)
		vPtr->name[i] = ' ';
}

//--== setType() ==------------------------------------------------------------
// Copy name into char* list node member
void StarRecords::setType(VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 8; i++)
		vPtr->type[i] = vPtr->record[i + 22];

	vPtr->type[8] = '\0';
}

//--== setRange() ==-----------------------------------------------------------
// Copy name into char* list node member
void StarRecords::setRange(VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 14; i++)
		vPtr->range[i] = vPtr->record[i + 32];

	vPtr->range[14] = '\0';
}

//--== setMaxMag() ==----------------------------------------------------------
// Parse chars into floating point list node member
void StarRecords::setMaxMag(VarStarRecord* vPtr)
{
		vPtr->max_mag = 0;

		if(isdigit(vPtr->record[33]))
			vPtr->max_mag +=
			(static_cast<float>(vPtr->record[33]) - 48) * 10;
		if(isdigit(vPtr->record[34]))
			vPtr->max_mag +=
			(static_cast<float>(vPtr->record[34]) - 48);
		if(isdigit(vPtr->record[36]))
			vPtr->max_mag +=
			(static_cast<float>(vPtr->record[36]) - 48) * 0.1f;
		if(isdigit(vPtr->record[37]))
			vPtr->max_mag +=
			(static_cast<float>(vPtr->record[37]) - 48) * 0.01f;
}

//--== setMinMag() ==----------------------------------------------------------
// Parse chars into floating point list node member
void StarRecords::setMinMag(VarStarRecord* vPtr)
{
		vPtr->min_mag = 0;

		if(isdigit(vPtr->record[40]))
			vPtr->min_mag +=
			(static_cast<float>(vPtr->record[40]) - 48) * 10;
		if(isdigit(vPtr->record[41]))
			vPtr->min_mag +=
			(static_cast<float>(vPtr->record[41]) - 48);
		if(isdigit(vPtr->record[43]))
			vPtr->min_mag +=
			(static_cast<float>(vPtr->record[43]) - 48) * 0.1f;
		if(isdigit(vPtr->record[44]))
			vPtr->min_mag +=
			(static_cast<float>(vPtr->record[44]) - 48) * 0.01f;
}

//--== setDays() ==------------------------------------------------------------
// Parse chars into floating point list node member
void StarRecords::setDays(VarStarRecord* vPtr)
{
		vPtr->period_days = 0;

		if(isdigit(vPtr->record[46]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[46]) - 48) * 1000;
		if(isdigit(vPtr->record[47]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[47]) - 48) * 100;
		if(isdigit(vPtr->record[48]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[48]) - 48) * 10;
		if(isdigit(vPtr->record[49]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[49]) - 48);
		if(vPtr->record[50] == 'K')
			vPtr->period_days *= 1000;
		if(isdigit(vPtr->record[51]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[51]) - 48) * 0.1;
		if(isdigit(vPtr->record[52]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[52]) - 48) * 0.01;
		if(isdigit(vPtr->record[53]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[53]) - 48) * 0.001;
		if(isdigit(vPtr->record[54]))
			vPtr->period_days +=
			(static_cast<double>(vPtr->record[54]) - 48) * 0.0001;
}

//--== setDesc() ==------------------------------------------------------------
// Copy name into char* list node member
void StarRecords::setDesc(VarStarRecord* vPtr)
{
	int i;

	for(i = 0; i < 20; i++)
		vPtr->desc[i] = vPtr->record[i + 55];

	vPtr->desc[20] = '\0';
}

//--== searchRecords() ==------------------------------------------------------
//
// Calls specific or non-specific search & sets search state vars
// When a record meets search criteria, a results node is created
//
int StarRecords::searchRecords(bool specific, bool magnitude, bool period)
{
	if(!is_loaded)
		return -1;

	if(is_searched)
	{
		delResults();
		is_searched = false;
		is_specific = false;
	}

	// most of the time, there will be at least one
	// result so create first results node and create a
	// record pointer to walk through the main list
	VarStarRecord* recPtr = topPtr;

	int i = 0;

   	// assign result node values until reaching
   	// the last main list node
	while(recPtr /*->pNext*/ != NULL)
	{
		// call specific or non-specific search
		if( specific ? isRecordInSearch(recPtr, magnitude, period)
			: isRecordInSearchUncertain(recPtr) )
		{
			if(i == 0)
			{
				topResPtr = new VarStarResult;
				resPtr = topResPtr;
				resPtr->pPrev = NULL;
			}
			else
			{
				resPtr->pNext = new VarStarResult;
				resPtr->pNext->pPrev = resPtr;
				resPtr = resPtr->pNext;
			}

			resPtr->recnum = ++i;
			resPtr->pVSR   = recPtr;
		}

		recPtr = recPtr->pNext;
	}

	// if any results, assign values to final result
	// node and set pointer to top
	if(i > 0)
	{
		// assign values to the final result node
		resPtr->pNext  = NULL;

		// reset search values to beginning of results
		lastResPtr = resPtr;
		resPtr     = topResPtr;
		begin  = true;
		end    = false;
		is_searched = true;
		specific == true ? is_specific = true : is_specific = false;
		
		return i;
	}
	else // no results, so clean up
	{
		//delete resPtr;

		resPtr     = NULL;
		topResPtr  = NULL;
		lastResPtr = NULL;
		begin  = true;
		end    = true;
		is_searched = false;
		is_specific = false;
		
		return 0;
	}
}

//--== clearSearch() ==--------------------------------------------------------
//
// Destroy search result list, if any,
// and create a result list including all records
//
void StarRecords::clearSearch()
{
	if(!is_loaded)
		return;

	if(is_searched)
	{
		delResults();
		is_searched = false;
		is_specific = false;
	}

	// if file loaded, there will be at least one
	// so create first results node and create a
	// record pointer to walk through the main list
	topResPtr = new VarStarResult;
	topResPtr->pPrev = NULL;
	resPtr = topResPtr;
	VarStarRecord* recPtr = topPtr;

	int i = 0;

   	// assign result node values until reaching
   	// the last main list node
	while(recPtr->pNext != NULL)
	{
		resPtr->recnum = ++i;
		resPtr->pVSR   = recPtr;
		resPtr->pNext  = new VarStarResult;
		resPtr->pNext->pPrev = resPtr;
		resPtr         = resPtr->pNext;

		recPtr = recPtr->pNext;
	}

	// assign values to the final result node
	resPtr->recnum = ++i;
	resPtr->pVSR   = recPtr;
	resPtr->pNext  = NULL;

	// reset search values to beginning of results
	lastResPtr = resPtr;
	resPtr = topResPtr;
	begin  = true;
	end    = false;
	is_searched = true;
	is_specific = false;
}

//--== isRecordInSearch() ==---------------------------------------------------
//
// Check if record meets "specific" search criteria
//
bool StarRecords::isRecordInSearch(VarStarRecord* vPtr, bool magnitude, bool period)
{
	// R.A. hours will "bridge" 0 hours and look from east to west
	// based on how both minimun and maximum are set no matter which
	// is the greater numeric value
	//
	// Max degrees is already enforced as greater than min degrees
	// with pos values being north and neg values being south
	//
	// Magnitude values NOT zero and...
	// Min magnitude >= value, so "no brighter than" (mag search tricky!)
	// Max magnitude <= value, so "no dimmer than"   (mag search tricky!)
	//
	// Period values NOT zero and...
	// Low and high limits on period in days
	if( (minHr <= maxHr
			? (vPtr->ra_hours >= minHr && vPtr->ra_hours <= maxHr)
			: (vPtr->ra_hours >= minHr || vPtr->ra_hours <= maxHr))
		&& vPtr->dec_degrees >= minDeg && vPtr->dec_degrees <= maxDeg
		&& ( magnitude
			? (	    vPtr->max_mag <= maxMag && vPtr->max_mag != 0
				&& (vPtr->min_mag >= minMag || vPtr->min_mag == 0))
			: true )
		&& ( period
			? (    vPtr->period_days >= minDays && vPtr->period_days <= maxDays
				&& vPtr->period_days != 0 )
			: true ) )

		return true;
	else
		return false;
}

//--== isRecordInSearchUncertain() ==------------------------------------------
//
// Check if record meets "sky area only" search criteria
//
bool StarRecords::isRecordInSearchUncertain(VarStarRecord* vPtr)
{
	// R.A. hours will "bridge" 0 hours and look from east to west
	// based on how both minimun and maximum are set no matter which
	// is the greater numeric value
	//
	// Max degrees is already enforced as greater than min degrees
	// with pos values being north and neg values being south
	//
	// NO magnitude or period information for records in this search
	// being mutually exclusive to the "specific" search within the
	// same sky area
	if( (minHr <= maxHr
			? (vPtr->ra_hours >= minHr  && vPtr->ra_hours <= maxHr)
			: (vPtr->ra_hours >= minHr  || vPtr->ra_hours <= maxHr))
		&& vPtr->dec_degrees >= minDeg  && vPtr->dec_degrees <= maxDeg
		&& vPtr->min_mag == 0
		&& vPtr->max_mag == 0
		&& vPtr->period_days == 0)

		return true;
	else
		return false;
}

//--== getHour() ==------------------------------------------------------------
// Return R.A. hour of current record or 0 if no results list
short int StarRecords::getHour() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->ra_hours;
	else
		return 0;
}

//--== getMin() ==-------------------------------------------------------------
// Return R.A. minute of current record or 0 if no results list
short int StarRecords::getMin () const
{
	if(resPtr != NULL)
		return resPtr->pVSR->ra_minutes;
	else
		return 0;
}

//--== getDeg() ==-------------------------------------------------------------
// Return declination degree of current record or 0 if no results list
short int StarRecords::getDeg () const
{
	if(resPtr != NULL)
		return resPtr->pVSR->dec_degrees;
	else
		return 0;
}

//--== getName() ==------------------------------------------------------------
// Return name of current record or empty string if no results list
const char* StarRecords::getName() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->name;
	else
		return "";
}

//--== getType() ==------------------------------------------------------------
// Return type of current record or empty string if no results list
const char* StarRecords::getType() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->type;
	else
		return "";
}

//--== getRange() ==-----------------------------------------------------------
// Return range string of current record or empty string if no results list
const char* StarRecords::getRange() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->range;
	else
		return "";
}

//--== getDays() ==------------------------------------------------------------
// Return period in days of current record or 0 if no results list
double StarRecords::getDays() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->period_days;
	else
		return 0;
}

//--== getDesc() ==------------------------------------------------------------
// Return description of current record or empty string if no results list
const char* StarRecords::getDesc() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->desc;
	else
		return "";
}

//--== getRange() ==-----------------------------------------------------------
// Return raw record of current record or empty string if no results list
const char* StarRecords::getRecord() const
{
	if(resPtr != NULL)
		return resPtr->pVSR->record;
	else
		return "";
}

//--== moveFirst() ==----------------------------------------------------------
// Move results pointer to the top of the results list
void StarRecords::moveFirst()
{
	if(!is_searched)
		return;
	resPtr = topResPtr;
	begin = true;
	end = false;
}

//--== movePrev() ==-----------------------------------------------------------
// Move results pointer to the previous node unless at the top
void StarRecords::movePrev()
{
	if(!is_searched)
		return;
	resPtr->pPrev == NULL ? begin = true : begin = false;
	if(resPtr->pPrev != NULL)
		resPtr = resPtr->pPrev;

	end = false;
}

//--== moveNext() ==-----------------------------------------------------------
// Move results pointer to the next node unless at the end
void StarRecords::moveNext()
{
	if(!is_searched)
		return;
	resPtr->pNext == NULL ? end = true : end = false;
	if(resPtr->pNext != NULL)
		resPtr = resPtr->pNext;

	begin = false;
}

//--== moveLast() ==-----------------------------------------------------------
// Move results pointer to the end of the results list
void StarRecords::moveLast()
{
	if(!is_searched)
		return;
	resPtr = lastResPtr;
	begin = false;
	end = true;

}

//--== bof() ==----------------------------------------------------------------
// Am I at the top of the results list?
bool StarRecords::bof() const
{
	return begin;
}

//--== eof() ==----------------------------------------------------------------
// Am I at the end of the results list?
bool StarRecords::eof() const
{
	return end;
}

//--== delResults() ==---------------------------------------------------------
//
// Destroy search result list if it exists
//
void StarRecords::delResults()
{
	VarStarResult* delResPtr = topResPtr;
	VarStarResult* recResPtr = topResPtr->pNext;

	// if top of list NULL, loop does not execute
	while(delResPtr != NULL)
	{
		delete delResPtr;

		// if only one result node, delete it,
		// NULL necessary pointers, quit
		if(recResPtr == NULL) 
		{
			delResPtr = NULL;

			topResPtr  = NULL;
			resPtr     = NULL;
			lastResPtr = NULL;

			break;
		}

		// if yet another node, move pointers
		if(recResPtr->pNext != NULL)
		{
			delResPtr = recResPtr;
			recResPtr = recResPtr->pNext;
		}
		else
		{
			delete recResPtr;
			delResPtr = NULL;
			recResPtr = NULL;

			topResPtr  = NULL;
			resPtr     = NULL;
			lastResPtr = NULL;
		}
	}
}

//--== delRecords() ==---------------------------------------------------------
//
// Destroy main records list if it exists
//
void StarRecords::delRecords()
{
	VarStarRecord* delPtr = topPtr;
	VarStarRecord* recPtr = topPtr->pNext;

	// if top of list NULL, loop does not execute
	while(delPtr != NULL)
	{
		delete delPtr;

		// if only one record node, delete it,
		// NULL necessary pointers, quit
		if(recPtr == NULL) 
		{
			delPtr = NULL;

			topPtr  = NULL;
			lastPtr = NULL;

			break;
		}

		// if yet another node, move pointers
		if(recPtr->pNext != NULL)
		{
			delPtr = recPtr;
			recPtr = recPtr->pNext;
		}
		else
		{
			delete recPtr;
			delPtr = NULL;
			recPtr = NULL;

			topPtr  = NULL;
			lastPtr = NULL;
		}
	}
}

//--== operator=() ==----------------------------------------------------------
//
// Overloaded assignment operator
//
StarRecords& StarRecords::operator=(const StarRecords& that)
{
	if(this != &that)
		deepCopy(that, false);

	return *this;
}

//--== StarRecords() ==--------------------------------------------------------
//
// Copy constructor
//
StarRecords::StarRecords(const StarRecords& that)
{
	if(this != &that)
		deepCopy(that);
}

//--== deepCopy() ==-----------------------------------------------------------
//
// Handles reloading the file into the new object, if loaded and
// recreating the search results in the new object based on the same criteria
//
void StarRecords::deepCopy(const StarRecords& that, bool i_am_new)
{
	this->minHr   = that.minHr;
	this->maxHr   = that.maxHr;
	this->minDeg  = that.minDeg;
	this->maxDeg  = that.maxDeg;
	this->minDays = that.minDays;
	this->maxDays = that.maxDays;
	this->minMag  = that.minMag;
	this->maxMag  = that.maxMag;

	strcpy(this->filename, that.filename);
	strcpy(this->rptname,  that.rptname);

	if(!i_am_new)
	{
		if(this->is_searched)
			this->delResults();
		if(this->is_loaded)
			this->delRecords();
	}

	this->topPtr     = NULL;
	this->lastPtr    = NULL;
	this->topResPtr  = NULL;
	this->resPtr     = NULL;
	this->lastResPtr = NULL;

	this->is_loaded   = false;
	this->is_searched = false;
	this->is_specific = false;
	this->begin	  = false;
	this->end	  = false;

	if(that.is_loaded)
	{
		this->loadRecords(this->filename);

		if(that.is_searched)
		{
			this->is_specific = that.is_specific;
			this->searchRecords(this->is_specific);
			this->moveFirst();
		}
	}
}

//--== ~StarRecords() ==-------------------------------------------------------
//
// Destructor tests for any necessary stream closing or dynamic mem cleanup
//
StarRecords::~StarRecords()
{
	if(starFile.is_open())
		starFile.close();
	if(starOut.is_open())
		starOut.close();
	if(is_searched)
		delResults();
	if(is_loaded)
		delRecords();
}
