/*
 * VarStar - Search & report variable star records in AAVSO data file
 * Copyright (C) 2005-2007  Frederick (Rick) Burton
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA.
 */

// StarRecords interface
//
// Loads vallongtype.txt into memory allowing searching by multiple
// criteria. This file is a public domain file downloaded from
// AAVSO at http://www.aavso.org

#ifndef CSTARRECORDS_H
#define CSTARRECORDS_H

#include <fstream>
using namespace std;

class StarRecords
{
public:
	// Constructor
	StarRecords();

	// File loading and saving 
	// ------------------------------------------------------------------------
	int loadRecords(const char* fname);
	int saveReport (const char* fname);
	int saveFile   (const char* fname);

	// Mutators
	void setSchMinHr  (short int min_ra_hrs);
	void setSchMaxHr  (short int max_ra_hrs);
	void setSchMinDeg (short int min_dec_deg);
	void setSchMaxDeg (short int max_dec_deg);
	void setSchMinDays(double min_days);
	void setSchMaxDays(double max_days);
	void setSchMinMag (double min_mag);
	void setSchMaxMag (double max_mag);

	// Accessors
	short int getSchMinHr  () const;
	short int getSchMaxHr  () const;
	short int getSchMinDeg () const;
	short int getSchMaxDeg () const;
	double    getSchMinDays() const;
	double    getSchMaxDays() const;
	float     getSchMinMag () const;
	float     getSchMaxMag () const;

	// Searching
	// v1.01 Enhanced search
	int searchRecords(bool specific = true, bool magnitude = true, bool period = true);
	void clearSearch(); // Resets to all records

	// Field value accessors
	short int   getHour  () const;
	short int   getMin   () const;
	short int   getDeg   () const;
	const char* getName  () const;
	const char* getType  () const;
	const char* getRange () const;
	double      getDays  () const;
	const char* getDesc  () const;
	const char* getRecord() const;

	// Get state: data loaded?
	bool isLoaded() const {return is_loaded;}

	// Record navigation 
	void moveFirst();
	void movePrev();
	void moveNext();
	void moveLast();
	bool bof() const;
	bool eof() const;

	// Memory management
	StarRecords& operator=(const StarRecords& that); // overloaded assignment
	StarRecords(const StarRecords& that);            // copy constructor
	~StarRecords();                                  // destructor
protected:
	// Variable star record node
	struct VarStarRecord
	{
		int       recnum;
		char      record[81];
		short int ra_hours;
		short int ra_minutes;
		short int dec_degrees;
		char      name[12];
		char      type[9];
		char      range[15];
		char      desc[21];
		double    max_mag;
		double    min_mag;
		double    period_days;

		VarStarRecord* pNext;
	};
	// Search result node
	struct VarStarResult
	{
		int recnum;

		VarStarResult* pPrev;
		VarStarRecord* pVSR;
		VarStarResult* pNext;
	};

	// I/O streams & file names
	char filename[256]; // Database file name
	char rptname[256];  // Output file name
	ifstream starFile;  // Database file handle
	ofstream starOut;   // Report file handle

	// Linked list navigation pointers
	VarStarRecord* topPtr;
	VarStarRecord* lastPtr;

	VarStarResult* topResPtr;
	VarStarResult* resPtr;
	VarStarResult* lastResPtr;

	// Search criteria
	short int minHr;
	short int maxHr;
	short int minDeg;
	short int maxDeg;
	double minDays;
	double maxDays;
	float minMag;
	float maxMag;

	// Record loading and searching
	bool isLineValidRecord(const char* testStr);                             // validate at load
	bool isRecordInSearchUncertain(VarStarRecord* vPtr);                     // non-specific search
	bool isRecordInSearch(VarStarRecord* vPtr, bool magnitude, bool period); // specific search
	                                                                         // v1.01 Enhanced search

	// Object state variables
	bool is_loaded;
	bool is_searched;
	bool is_specific;
	bool begin;
	bool end;

	// Internal parsing mutators for star record fields
	void setHours  (VarStarRecord* vPtr);
	void setMinutes(VarStarRecord* vPtr);
	void setDegrees(VarStarRecord* vPtr);
	void setName   (VarStarRecord* vPtr);
	void setType   (VarStarRecord* vPtr);
	void setRange  (VarStarRecord* vPtr);
	void setMaxMag (VarStarRecord* vPtr);
	void setMinMag (VarStarRecord* vPtr);
	void setDays   (VarStarRecord* vPtr);
	void setDesc   (VarStarRecord* vPtr);

	// Memeber copy handler for assignment op and copy const
	void deepCopy(const StarRecords& that, bool i_am_new = true);

	// Linked list destroyers
	void delRecords();
	void delResults();
};

#endif //CSTARRECORDS_H
